bison-patches
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

17-fyi-escape-tags.patch


From: Akim Demaille
Subject: 17-fyi-escape-tags.patch
Date: Thu, 27 Dec 2001 19:02:10 +0100

Index: ChangeLog
from  Akim Demaille  <address@hidden>
        * src/print.c, src/print_graph.c (escape): New.
        Use it to quote the TAGS outputs.
        * src/print_graph.c (print_state): Now errors are in red, and
        reductions in green.
        Prefer high to wide: output the state number on a line of its own.
        
        
Index: src/print.c
--- src/print.c Thu, 27 Dec 2001 11:15:14 +0100 akim
+++ src/print.c Thu, 27 Dec 2001 11:43:52 +0100 akim
@@ -20,6 +20,7 @@
 
 
 #include "system.h"
+#include "quotearg.h"
 #include "files.h"
 #include "gram.h"
 #include "LR0.h"
@@ -43,6 +44,19 @@
 }
 #endif
 
+static inline const char *
+escape (const char *s)
+{
+  return quotearg_n_style (1, escape_quoting_style, s);
+}
+
+/* Be cautious not to use twice the same slot in a single expression. */
+static inline const char *
+escape2 (const char *s)
+{
+  return quotearg_n_style (2, escape_quoting_style, s);
+}
+
 
 /*--------------------------------.
 | Report information on a state.  |
@@ -78,15 +92,15 @@
            sp++;
 
          rule = -(*sp);
-         fprintf (out, "    %s  ->  ", tags[rule_table[rule].lhs]);
+         fprintf (out, "    %s  ->  ", escape (tags[rule_table[rule].lhs]));
 
          for (sp = ritem + rule_table[rule].rhs; sp < sp1; sp++)
-           fprintf (out, "%s ", tags[*sp]);
+           fprintf (out, "%s ", escape (tags[*sp]));
 
          fputc ('.', out);
 
          for (/* Nothing */; *sp > 0; ++sp)
-           fprintf (out, " %s", tags[*sp]);
+           fprintf (out, " %s", escape (tags[*sp]));
 
          fprintf (out, _("   (rule %d)"), rule);
          fputc ('\n', out);
@@ -110,7 +124,7 @@
        int symbol = state_table[state1]->accessing_symbol;
        fprintf (out,
                 _("    %-4s\tshift, and go to state %d\n"),
-                tags[symbol], state1);
+                escape (tags[symbol]), state1);
       }
 
   if (i > 0)
@@ -127,7 +141,7 @@
   for (i = 0; i < errp->nerrs; ++i)
     if (errp->errs[i])
       fprintf (out, _("    %-4s\terror (nonassociative)\n"),
-              tags[errp->errs[i]]);
+              escape (tags[errp->errs[i]]));
 
   if (i > 0)
     fputc ('\n', out);
@@ -151,7 +165,7 @@
            int state1 = shiftp->shifts[i];
            int symbol = state_table[state1]->accessing_symbol;
            fprintf (out, _("    %-4s\tgo to state %d\n"),
-                    tags[symbol], state1);
+                    escape (tags[symbol]), state1);
          }
 
       fputc ('\n', out);
@@ -175,7 +189,7 @@
       int rule = redp->rules[0];
       int symbol = rule_table[rule].lhs;
       fprintf (out, _("    $default\treduce using rule %d (%s)\n\n"),
-              rule, tags[symbol]);
+              rule, escape (tags[symbol]));
       return;
     }
 
@@ -207,11 +221,11 @@
       for (i = 0; i < ntokens; i++)
        if (BITISSET (lookaheadset, i))
          fprintf (out, _("    %-4s\t[reduce using rule %d (%s)]\n"),
-                  tags[i], default_rule,
-                  tags[rule_table[default_rule].lhs]);
+                  escape (tags[i]), default_rule,
+                  escape2 (tags[rule_table[default_rule].lhs]));
 
       fprintf (out, _("    $default\treduce using rule %d (%s)\n\n"),
-              default_rule, tags[rule_table[default_rule].lhs]);
+              default_rule, escape (tags[rule_table[default_rule].lhs]));
     }
   else if (state->nlookaheads >= 1)
     {
@@ -265,9 +279,9 @@
                      if (state->lookaheadsp + j != default_LA)
                        fprintf (out,
                                 _("    %-4s\treduce using rule %d (%s)\n"),
-                                tags[i],
+                                escape (tags[i]),
                                 LAruleno[state->lookaheadsp + j],
-                                tags[rule_table[LAruleno[state->lookaheadsp + 
j]].lhs]);
+                                escape2 
(tags[rule_table[LAruleno[state->lookaheadsp + j]].lhs]));
                      else
                        defaulted = 1;
 
@@ -278,15 +292,15 @@
                      if (defaulted)
                        fprintf (out,
                                 _("    %-4s\treduce using rule %d (%s)\n"),
-                                tags[i],
+                                escape (tags[i]),
                                 LAruleno[default_LA],
-                                tags[rule_table[LAruleno[default_LA]].lhs]);
+                                escape2 
(tags[rule_table[LAruleno[default_LA]].lhs]));
                      defaulted = 0;
                      fprintf (out,
                               _("    %-4s\t[reduce using rule %d (%s)]\n"),
-                              tags[i],
+                              escape (tags[i]),
                               LAruleno[state->lookaheadsp + j],
-                              tags[rule_table[LAruleno[state->lookaheadsp + 
j]].lhs]);
+                              escape2 
(tags[rule_table[LAruleno[state->lookaheadsp + j]].lhs]));
                    }
                }
            }
@@ -294,7 +308,7 @@
 
       if (default_LA >= 0)
        fprintf (out, _("    $default\treduce using rule %d (%s)\n"),
-                default_rule, tags[rule_table[default_rule].lhs]);
+                default_rule, escape (tags[rule_table[default_rule].lhs]));
     }
 }
 
@@ -362,11 +376,11 @@
     if (rule_table[i].useful)
       {
        fprintf (out, _("  %3d %3d %s ->"),
-                i, rule_table[i].line, tags[rule_table[i].lhs]);
+                i, rule_table[i].line, escape (tags[rule_table[i].lhs]));
        rule = &ritem[rule_table[i].rhs];
        if (*rule > 0)
          while (*rule > 0)
-           fprintf (out, " %s", tags[*rule++]);
+           fprintf (out, " %s", escape (tags[*rule++]));
        else
          fprintf (out, " /* %s */", _("empty"));
        fputc ('\n', out);
@@ -376,14 +390,14 @@
 
   /* TERMINAL (type #) : rule #s terminal is on RHS */
   fprintf (out, "%s\n\n", _("Terminals, with rules where they appear"));
-  fprintf (out, "%s (-1)\n", tags[0]);
+  fprintf (out, "%s (-1)\n", escape (tags[0]));
 
   for (i = 0; i <= max_user_token_number; i++)
     if (token_translations[i] != 2)
       {
        buffer[0] = 0;
-       column = strlen (tags[token_translations[i]]);
-       fputs (tags[token_translations[i]], out);
+       column = strlen (escape (tags[token_translations[i]]));
+       fputs (escape (tags[token_translations[i]]), out);
        END_TEST (50);
        sprintf (buffer, " (%d)", i);
 
@@ -418,8 +432,8 @@
        }
 
       buffer[0] = 0;
-      fputs (tags[i], out);
-      column = strlen (tags[i]);
+      fputs (escape (tags[i]), out);
+      column = strlen (escape (tags[i]));
       sprintf (buffer, " (%d)", i);
       END_TEST (0);
 
Index: src/print_graph.c
--- src/print_graph.c Wed, 26 Dec 2001 23:00:34 +0100 akim
+++ src/print_graph.c Thu, 27 Dec 2001 11:37:01 +0100 akim
@@ -19,6 +19,7 @@
    Boston, MA 02111-1307, USA.  */
 
 #include "system.h"
+#include "quotearg.h"
 #include "files.h"
 #include "gram.h"
 #include "LR0.h"
@@ -36,6 +37,13 @@
 static graph_t graph;
 static FILE *fgraph = NULL;
 
+static inline const char *
+escape (const char *s)
+{
+  return quotearg_n_style (1, escape_quoting_style, s);
+}
+
+
 /* This part will construct the label of nodes. */
 static void
 print_core (state_t *state, struct obstack *node_obstack)
@@ -49,7 +57,7 @@
   sitems = itemset;
   snitems = nitemset;
 
-  obstack_fgrow1 (node_obstack, "%2d: ", state->number);
+  obstack_fgrow1 (node_obstack, "state %2d\n", state->number);
   for (i = 0; i < snitems; i++)
     {
       short *sp;
@@ -64,17 +72,17 @@
       rule = -(*sp);
 
       if (i)
-       obstack_sgrow (node_obstack, "\n    ");
+       obstack_1grow (node_obstack, '\n');
       obstack_fgrow1 (node_obstack, " %s -> ",
-                     tags[rule_table[rule].lhs]);
+                     escape (tags[rule_table[rule].lhs]));
 
       for (sp = ritem + rule_table[rule].rhs; sp < sp1; sp++)
-       obstack_fgrow1 (node_obstack, "%s ", tags[*sp]);
+       obstack_fgrow1 (node_obstack, "%s ", escape (tags[*sp]));
 
       obstack_1grow (node_obstack, '.');
 
       for (/* Nothing */; *sp > 0; ++sp)
-       obstack_fgrow1 (node_obstack, " %s", tags[*sp]);
+       obstack_fgrow1 (node_obstack, " %s", escape (tags[*sp]));
     }
 }
 
@@ -113,9 +121,12 @@
        edge.sourcename = node_name;
        sprintf (buff, "%d", state1);
        edge.targetname = buff;
-       /* Shifts are blue, gotos are red. */
-       edge.color = SHIFT_IS_SHIFT(shiftp, i) ? blue : red;
-       edge.label = tags[symbol];
+       /* Shifts are blue, gotos are green, and error is red. */
+       if (SHIFT_IS_ERROR (shiftp, i))
+         edge.color = red;
+       else
+         edge.color = SHIFT_IS_SHIFT(shiftp, i) ? blue : green;
+       edge.label = escape (tags[symbol]);
        output_edge (&edge, fgraph);
        close_edge (fgraph);
       }



reply via email to

[Prev in Thread] Current Thread [Next in Thread]